'------------------------------------------------------------------------------
' 
' 	BSD License
'   -----------
' 
'   Copyright (c) 2012 Elba Corp.  All rights reserved.
' 
'      Elba Corp.
'      10915 SW Avocet Ct.
'      Beaverton, OR  97007
' 
'   Redistribution and use in source and binary forms, with or without
'   modification, are permitted only for use by licensees of the ZBasic
'   Compiler for generic AVR devices and only then if all the following
'   conditions are met:
' 
'   - Redistributions of source code must retain the above copyright notice,
'     this list of conditions and the following disclaimer.
' 
'   - Redistributions in binary form must reproduce the above copyright notice,
'     this list of conditions and the following disclaimer in the documentation
'     and/or other materials provided with the distribution.
' 
'   - Neither the name of the copyright owner nor the names of its contributors
'     may be used to endorse or promote products derived from this software
'     without specific prior written permission of the copyright owner.
' 
'   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
'   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
'   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
'   ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
'   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
'   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
'   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
'   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
'   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
'   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
'   POSSIBILITY OF SUCH DAMAGE.
' 
'------------------------------------------------------------------------------

'------------------------------------------------------------------------------
' 
' The code in this module illustrates how to use the DS3234 timekeeping chip.
' 
'------------------------------------------------------------------------------

' configure these values to meet your needs
Private Const csPin as Byte = A.2		' set the pin for the DS3234 chip select
Private Const chan as Byte = 1			' set the SPI channel to be used

' uncomment the following line to include a simple test driver
#define INCLUDE_TEST_DRIVER

' This workaround addresses a problem in the v4.0.0 ZBasic compiler.
' It isn't needed for earlier versions nor for v4.0.2 and later versions
' but is harmless in either case.
#c
	#if !defined(zbfracf)
	#define zbfracf zbFracf
	#endif
#endc

'------------------------------------------------------------------------------
' Structure to hold time/date information.  The 'fraction' field is an
' integral value representing a fraction of a second in units of 1/RTC Tick Frequency.
' The 'year' field is the actual year, e.g. 2012.  The remaining fields should
' be self-explanatory.
Public Structure TimeDate
	Dim year as UnsignedInteger
	Dim month as Byte
	Dim day as Byte
	Dim hour as Byte
	Dim minute as Byte
	Dim second as Byte
	Dim fraction as UnsignedInteger
End Structure

'------------------------------------------------------------------------------
'
'' InitDS3234
'
' Initialize the timekeeping chip.  The oscillator is enabled as is 1Hz output.
'
Sub InitDS3234()
	Dim odata(1 to 3) as Byte
	odata(1) = &H8e
	odata(2) = &H60
	odata(3) = &H08
	Call SPICmd(chan, 1, odata, 0, 0)
	Call Sleep(0.2)
End Sub

'------------------------------------------------------------------------------
'
'' GetTimeDS3234
'
' Populate the passed structure using current DS3234 time.
'
Sub GetTimeDS3234(ByRef td as TimeDate)
	Dim odata as Byte
	Dim idata(1 to 7) as Byte

	' read the data from the DS3234
	odata = &H00
	Call SPICmd(chan, SizeOf(odata), odata, SizeOf(idata), idata)

	' convert the data from DS3234 format
	td.fraction = 0
	td.second = fromBCD(idata(1))
	td.minute = fromBCD(idata(2))
	td.hour = fromBCD(idata(3))
	td.day = fromBCD(idata(5))
	td.month = fromBCD(idata(6))
	td.year = CUInt(fromBCD(idata(7))) + 2000
End Sub

'------------------------------------------------------------------------------
'
'' SetZBasicDS3234
'
' Set the DS3234 time with data from the passed structure.
'
Sub SetTimeDS3234(ByVal td as TimeDate)
	Dim odata(1 to 8) as Byte

	' prepare the data to write to the DS3234
	odata(1) = &H80
	odata(2) = toBCD(td.second)
	odata(3) = toBCD(td.minute)
	odata(4) = toBCD(td.hour)
	odata(5) = 0
	odata(6) = toBCD(td.day)
	odata(7) = toBCD(td.month)
	odata(8) = toBCD(LoByte(td.year - 2000))

	' write the data to the DS3234
	Call SPICmd(chan, SizeOf(odata), odata, 0, 0)
End Sub

#if Option.RTCEnabled

'------------------------------------------------------------------------------
'
'' GetTimeZBasic
'
' Populate the passed structure using current ZBasic time.
'
Sub GetTimeZBasic(ByRef td as TimeDate)
	Dim sec as Single
	Call GetTime(td.hour, td.minute, sec)
	td.second = CByte(Fix(sec))
	td.fraction = CUInt(Fraction(sec) * CSng(Register.RTCTickFrequency))
	Call GetDate(td.year, td.month, td.day)
End Sub

'------------------------------------------------------------------------------
'
'' SetTimeZBasic
'
' Set the ZBasic time with data in the passed structure.
'
Sub SetTimeZBasic(ByRef td as TimeDate)
	Dim sec as Single
	sec = CSng(td.second) + CSng(td.fraction) / CSng(Register.RTCTickFrequency)
	Call PutTime(td.hour, td.minute, sec)
	Call PutDate(td.year, td.month, td.day)
End Sub

#endif

'------------------------------------------------------------------------------
'
'' toBCD
'
' This function converts a byte value to packed BCD form with one decimal digit in
' the most significant nibble and one decimal digit in the lease significant nibble.
'
Private Function toBCD(ByVal val as Byte) as Byte
	val = val Mod 100
	toBCD = Shl(val \ 10, 4) Or (val Mod 10)
End Function

'------------------------------------------------------------------------------
'
'' fromBCD
'
' This function converts a packed BCD value (representing two decimal digits) to
' the equivalent byte value.
'
Private Function fromBCD(ByVal val as Byte) as Byte
	fromBCD = (Shr(val, 4) * 10) +  (val And &H0f)
End Function

'------------------------------------------------------------------------------

#if defined(INCLUDE_TEST_DRIVER)

Sub Main()
	Dim td as TimeDate
	Call OpenSPI(chan, &H05, csPin)
	Call InitDS3234()

#if 0
	' set the time on the DS3234
	td.year = 2012
	td.month = 5
	td.day = 7
	td.hour = 15
	td.minute = 03
	td.second = 45
	Call SetTimeDS3234(td)
#endif

	' retrieve the current time from the DS3234 and set the ZBasic time
	Call GetTimeDS3234(td)
#if Option.RTCEnabled
	Call SetTimeZBasic(td)
#endif

	' display the date/time repeatedly
	Do
#if Option.RTCEnabled
		Call GetTimeZBasic(td)
#else
		Call GetTimeDS3234(td)
#endif
		Debug.Print td.year; "/";
		Call outputDigits(td.month, "/")
		Call outputDigits(td.day, " ")
		Call outputDigits(td.hour, ":")
		Call outputDigits(td.minute, ":")
		Call outputDigits(td.second, "")
		Debug.Print
		Call Sleep(5.0)
	Loop
End Sub

'------------------------------------------------------------------------------
'
'' outputDigits
'
' Output two digits corresponding to a passed value.
'
Private Sub outputDigits(ByVal val as Byte, ByVal sep as String)
	Dim digit1 as Byte
	Dim digit2 as Byte
	digit2 = (val Mod 10)
	val = val \ 10
	digit1 = (val Mod 10)
	Debug.Print digit1; digit2; sep;
End Sub
#endif
